@@ -6,51 +6,34 @@ import os
 from dataclasses import dataclass
 from enum import Enum
 from pathlib import Path
-from typing import Dict, List, Optional, Union
 
 import netCDF4
 import numpy as np
 from numpy import ma
 
 from . import utils
+from .variables import VARIABLES, Product
 from .version import __version__
 
 FILE_PATH = os.path.dirname(os.path.realpath(__file__))
 
 METADATA_CONFIG = utils.read_config(f"{FILE_PATH}/metadata_config.ini")
 DATA_CONFIG = utils.read_config(f"{FILE_PATH}/data_quality_config.ini")
-
-
-class Product(str, Enum):
-    # Level 1b
-    RADAR = "radar"
-    LIDAR = "lidar"
-    MWR = "mwr"
-    DISDROMETER = "disdrometer"
-    MODEL = "model"
-    # Level 1c
-    CATEGORIZE = "categorize"
-    # Level 2
-    CLASSIFICATION = "classification"
-    IWC = "iwc"
-    LWC = "lwc"
-    DRIZZLE = "drizzle"
-    # Experimental
-    DER = "der"
-    IER = "ier"
+CF_AREA_TYPES_XML = f"{FILE_PATH}/area-type-table.xml"
+CF_STANDARD_NAMES_XML = f"{FILE_PATH}/cf-standard-name-table.xml"
+CF_REGION_NAMES_XML = f"{FILE_PATH}/standardized-region-list.xml"
 
 
 class ErrorLevel(str, Enum):
-    PASS = "pass"
     WARNING = "warning"
     ERROR = "error"
+    INFO = "info"
 
 
 @dataclass
 class TestReport:
     testId: str
-    description: str
-    exceptions: List[dict]
+    exceptions: list[dict]
 
     def values(self):
         return {
@@ -64,14 +47,16 @@ class TestReport:
 class FileReport:
     timestamp: str
     qcVersion: str
-    tests: List[Dict]
+    tests: list[dict]
 
 
 def run_tests(
-    filename: Path,
-    cloudnet_file_type: Optional[str] = None,
-    ignore_tests: Optional[List[str]] = None,
+    filename: Path | str,
+    cloudnet_file_type: str | None = None,
+    ignore_tests: list[str] | None = None,
 ) -> dict:
+    if isinstance(filename, str):
+        filename = Path(filename)
     with netCDF4.Dataset(filename) as nc:
         if cloudnet_file_type is None:
             try:
@@ -84,7 +69,7 @@ def run_tests(
                 return {}
         logging.debug(f"Filename: {filename.stem}")
         logging.debug(f"File type: {cloudnet_file_type}")
-        test_reports: List[Dict] = []
+        test_reports: list[dict] = []
         for cls in Test.__subclasses__():
             if ignore_tests and cls.__name__ in ignore_tests:
                 continue
@@ -94,8 +79,8 @@ def run_tests(
                 for exception in test_instance.report.values()["exceptions"]:
                     assert exception["result"] in (
                         ErrorLevel.ERROR,
-                        ErrorLevel.PASS,
                         ErrorLevel.WARNING,
+                        ErrorLevel.INFO,
                     )
                 test_reports.append(test_instance.report.values())
     return FileReport(
@@ -106,15 +91,17 @@ def run_tests(
 
 
 def test(
+    name: str,
     description: str,
-    error_level: Optional[ErrorLevel] = None,
-    products: Optional[List[Product]] = None,
-    ignore_products: Optional[List[Product]] = None,
+    error_level: ErrorLevel | None = None,
+    products: list[Product] | None = None,
+    ignore_products: list[Product] | None = None,
 ):
     """Decorator for the tests."""
 
     def fun(cls):
 
+        setattr(cls, "name", name)
         setattr(cls, "description", description)
         if error_level is not None:
             setattr(cls, "severity", error_level)
@@ -131,9 +118,10 @@ def test(
 class Test:
     """Test base class."""
 
+    name: str
     description: str
     severity = ErrorLevel.WARNING
-    products: List[str] = [member.value for member in Product]  # All products by default
+    products: list[str] = Product.all()
 
     def __init__(self, nc: netCDF4.Dataset, filename: Path, cloudnet_file_type: str):
         self.filename = filename
@@ -141,22 +129,13 @@ class Test:
         self.cloudnet_file_type = cloudnet_file_type
         self.report = TestReport(
             testId=self.__class__.__name__,
-            description=self.description,
             exceptions=[],
         )
 
     def run(self):
         raise NotImplementedError
 
-    def _test_variable_attribute(self, attribute: str):
-        for key, expected in METADATA_CONFIG.items(attribute):
-            if key in self.nc.variables:
-                value = getattr(self.nc.variables[key], attribute, "")
-                if value != expected:
-                    msg = utils.create_expected_received_msg(key, expected, value)
-                    self._add_message(msg)
-
-    def _add_message(self, message: Union[str, list]):
+    def _add_message(self, message: str | list):
         self.report.exceptions.append(
             {
                 "message": utils.format_msg(message),
@@ -169,79 +148,35 @@ class Test:
         keys = METADATA_CONFIG[config_section][field].split(",")
         return np.char.strip(keys)
 
+    def _get_required_variables(self) -> dict:
+        return {
+            name: var
+            for name, var in VARIABLES.items()
+            if var.required is not None and self.cloudnet_file_type in var.required
+        }
 
-# ---------------------- #
-# ------ Warnings ------ #
-# ---------------------- #
+    def _get_required_variable_names(self) -> set:
+        required_variables = self._get_required_variables()
+        return set(required_variables.keys())
+
+    def _test_variable_attribute(self, attribute: str):
+        for key in self.nc.variables.keys():
+            if key not in VARIABLES:
+                continue
+            expected = getattr(VARIABLES[key], attribute)
+            if expected is not None:
+                value = getattr(self.nc.variables[key], attribute, "")
+                if value != expected:
+                    msg = utils.create_expected_received_msg(key, expected, value)
+                    self._add_message(msg)
 
 
-@test("Test that unit attribute of variable matches expected value")
-class TestUnits(Test):
-    def run(self):
-        self._test_variable_attribute("units")
+# --------------------#
+# ------ Infos ------ #
+# --------------------#
 
 
-@test(
-    "Test that long_name attribute of variable matches expected value",
-    ignore_products=[Product.MODEL],
-)
-class TestLongNames(Test):
-    def run(self):
-        self._test_variable_attribute("long_name")
-
-
-@test(
-    "Test that standard_name attribute of variable matches CF convention",
-    ignore_products=[Product.MODEL],
-)
-class TestStandardNames(Test):
-    def run(self):
-        self._test_variable_attribute("standard_name")
-
-
-@test("Find invalid data types")
-class TestDataTypes(Test):
-    def run(self):
-        for key in self.nc.variables:
-            expected = "float32"
-            received = self.nc.variables[key].dtype.name
-            for config_key, custom_value in METADATA_CONFIG.items("data_types"):
-                if config_key == key:
-                    expected = custom_value
-                    break
-            if received != expected:
-                if key == "time" and received in ("float32", "float64"):
-                    continue
-                msg = utils.create_expected_received_msg(key, expected, received)
-                self._add_message(msg)
-
-
-@test("Find missing global attributes")
-class TestGlobalAttributes(Test):
-    def run(self):
-        nc_keys = self.nc.ncattrs()
-        config_keys = self._read_config_keys("required_global_attributes")
-        missing_keys = list(set(config_keys) - set(nc_keys))
-        for key in missing_keys:
-            self._add_message(f"'{key}' is missing.")
-
-
-@test("Test median LWP value", ErrorLevel.WARNING, [Product.MWR, Product.CATEGORIZE])
-class TestMedianLwp(Test):
-    def run(self):
-        key = "lwp"
-        if key not in self.nc.variables:
-            self.severity = ErrorLevel.ERROR
-            self._add_message(f"'{key}' is missing.")
-            return
-        limits = [-0.5, 10]
-        median_lwp = ma.median(self.nc.variables[key][:]) / 1000  # g -> kg
-        if median_lwp < limits[0] or median_lwp > limits[1]:
-            msg = utils.create_out_of_bounds_msg(key, *limits, median_lwp)
-            self._add_message(msg)
-
-
-@test("Find suspicious data values")
+@test("Variable outliers", "Find suspicious data values.", error_level=ErrorLevel.INFO)
 class FindVariableOutliers(Test):
     def run(self):
         for key, limits_str in DATA_CONFIG.items("limits"):
@@ -261,43 +196,12 @@ class FindVariableOutliers(Test):
                     self._add_message(msg)
 
 
-@test("Find suspicious global attribute values")
-class FindAttributeOutliers(Test):
-    def run(self):
-        for key, limits_str in METADATA_CONFIG.items("attribute_limits"):
-            limits = [float(x) for x in limits_str.split(",")]
-            if hasattr(self.nc, key):
-                value = float(self.nc.getncattr(key))
-                if value < limits[0] or value > limits[1]:
-                    msg = utils.create_out_of_bounds_msg(key, *limits, value)
-                    self._add_message(msg)
-
-
-@test("Test that file contains data", ignore_products=[Product.MODEL])
-class TestDataCoverage(Test):
-    def run(self):
-        grid = ma.array(np.linspace(0, 24, int(24 * (60 / 5)) + 1))
-        time = self.nc["time"][:]
-        bins_with_no_data = 0
-        for ind, t in enumerate(grid[:-1]):
-            ind2 = np.where((time > t) & (time <= grid[ind + 1]))[0]
-            if len(ind2) == 0:
-                bins_with_no_data += 1
-        missing = bins_with_no_data / len(grid) * 100
-        if missing > 10:
-            self._add_message(f"{round(missing)}% of day's data is missing.")
-
-
-@test("Test that LDR values are proper", products=[Product.RADAR, Product.CATEGORIZE])
-class TestLDR(Test):
-    def run(self):
-        if "ldr" in self.nc.variables:
-            ldr = self.nc["ldr"][:]
-            if ldr.mask.all():
-                self._add_message("LDR exists but all the values are invalid.")
-
-
-@test("Test radar folding", products=[Product.RADAR, Product.CATEGORIZE])
+@test(
+    "Radar folding",
+    "Test for radar folding.",
+    products=[Product.RADAR, Product.CATEGORIZE],
+    error_level=ErrorLevel.INFO,
+)
 class FindFolding(Test):
     def run(self):
         key = "v"
@@ -311,10 +215,142 @@ class FindFolding(Test):
         difference = np.abs(np.diff(data, axis=1))
         n_suspicious = ma.sum(difference > v_threshold)
         if n_suspicious > 20:
-            self._add_message(f"{n_suspicious} suspicious range gates. Folding might be present.")
+            self._add_message(f"{n_suspicious} suspicious pixels. Folding might be present.")
 
 
-@test("Test if beta not range-corrected", products=[Product.LIDAR])
+@test(
+    "Data coverage",
+    "Test that file contains enough data.",
+    ignore_products=[Product.MODEL],
+    error_level=ErrorLevel.INFO,
+)
+class TestDataCoverage(Test):
+    def run(self):
+        grid = ma.array(np.linspace(0, 24, int(24 * (60 / 5)) + 1))
+        time = self.nc["time"][:]
+        bins_with_no_data = 0
+        for ind, t in enumerate(grid[:-1]):
+            ind2 = np.where((time > t) & (time <= grid[ind + 1]))[0]
+            if len(ind2) == 0:
+                bins_with_no_data += 1
+        missing = bins_with_no_data / len(grid) * 100
+        if missing > 20:
+            self._add_message(f"{round(missing)}% of day's data is missing.")
+
+
+@test(
+    "Variable names",
+    "Check that variables have expected names.",
+    ignore_products=[Product.MODEL],
+    error_level=ErrorLevel.INFO,
+)
+class TestVariableNamesDefined(Test):
+    def run(self):
+        for key in self.nc.variables.keys():
+            if key not in VARIABLES:
+                self._add_message(f"'{key}' is not defined.")
+
+
+# ---------------------- #
+# ------ Warnings ------ #
+# ---------------------- #
+
+
+@test("Units", "Check that variables have expected units.")
+class TestUnits(Test):
+    def run(self):
+        self._test_variable_attribute("units")
+
+
+@test(
+    "Long names",
+    "Check that variables have expected long names.",
+    ignore_products=[Product.MODEL],
+)
+class TestLongNames(Test):
+    def run(self):
+        self._test_variable_attribute("long_name")
+
+
+@test(
+    "Standard names",
+    "Check that variable have expected standard names.",
+    ignore_products=[Product.MODEL],
+)
+class TestStandardNames(Test):
+    def run(self):
+        self._test_variable_attribute("standard_name")
+
+
+@test("Data types", "Check that variables have expected data types.")
+class TestDataTypes(Test):
+    def run(self):
+        for key in self.nc.variables:
+            if key not in VARIABLES:
+                continue
+            expected = VARIABLES[key].dtype.value
+            received = self.nc.variables[key].dtype.name
+            if received != expected:
+                if key == "time" and received in ("float32", "float64"):
+                    continue
+                msg = utils.create_expected_received_msg(key, expected, received)
+                self._add_message(msg)
+
+
+@test("Global attributes", "Check that file contains required global attributes.")
+class TestGlobalAttributes(Test):
+    def run(self):
+        nc_keys = self.nc.ncattrs()
+        config_keys = self._read_config_keys("required_global_attributes")
+        missing_keys = list(set(config_keys) - set(nc_keys))
+        for key in missing_keys:
+            self._add_message(f"'{key}' is missing.")
+
+
+@test(
+    "Median LWP",
+    "Test that median of LWP values is within a reasonable range.",
+    ErrorLevel.WARNING,
+    [Product.MWR, Product.CATEGORIZE],
+)
+class TestMedianLwp(Test):
+    def run(self):
+        key = "lwp"
+        if key not in self.nc.variables:
+            self.severity = ErrorLevel.ERROR
+            self._add_message(f"'{key}' is missing.")
+            return
+        limits = [-0.5, 10]
+        median_lwp = ma.median(self.nc.variables[key][:]) / 1000  # g -> kg
+        if median_lwp < limits[0] or median_lwp > limits[1]:
+            msg = utils.create_out_of_bounds_msg(key, *limits, median_lwp)
+            self._add_message(msg)
+
+
+@test("Attribute outliers", "Find suspicious values in global attributes.")
+class FindAttributeOutliers(Test):
+    def run(self):
+        for key, limits_str in METADATA_CONFIG.items("attribute_limits"):
+            limits = [float(x) for x in limits_str.split(",")]
+            if hasattr(self.nc, key):
+                value = float(self.nc.getncattr(key))
+                if value < limits[0] or value > limits[1]:
+                    msg = utils.create_out_of_bounds_msg(key, *limits, value)
+                    self._add_message(msg)
+
+
+@test(
+    "LDR values", "Test that LDR values are proper.", products=[Product.RADAR, Product.CATEGORIZE]
+)
+class TestLDR(Test):
+    def run(self):
+        if "ldr" in self.nc.variables:
+            ldr = self.nc["ldr"][:]
+            if ldr.mask.all():
+                self._add_message("LDR exists but all the values are invalid.")
+
+
+@test("Range correction", "Test that beta is range corrected.", products=[Product.LIDAR])
 class TestIfRangeCorrected(Test):
     def run(self):
         try:
@@ -333,7 +369,8 @@ class TestIfRangeCorrected(Test):
 
 
 @test(
-    "Test that valid instrument PID exists",
+    "Instrument PID",
+    "Test that valid instrument PID exists.",
     ErrorLevel.WARNING,
     [Product.MWR, Product.LIDAR, Product.RADAR, Product.DISDROMETER],
 )
@@ -353,7 +390,7 @@ class TestInstrumentPid(Test):
 # -------------------- #
 
 
-@test("Test that time vector is continuous", ErrorLevel.ERROR)
+@test("Time vector", "Test that time vector is continuous.", ErrorLevel.ERROR)
 class TestTimeVector(Test):
     def run(self):
         time = self.nc["time"][:]
@@ -379,12 +416,12 @@ class TestTimeVector(Test):
             self._add_message(msg)
 
 
-@test("Find missing variables", ErrorLevel.ERROR)
+@test("Variables", "Check that file contains required variables.", ErrorLevel.ERROR)
 class TestVariableNames(Test):
     def run(self):
-        nc_keys = self.nc.variables.keys()
-        config_keys = self._read_config_keys("required_variables")
-        missing_keys = list(set(config_keys) - set(nc_keys))
+        keys_in_file = set(self.nc.variables.keys())
+        required_keys = self._get_required_variable_names()
+        missing_keys = list(required_keys - keys_in_file)
         for key in missing_keys:
             self._add_message(f"'{key}' is missing.")
 
@@ -394,26 +431,19 @@ class TestVariableNames(Test):
 # ----------------------------- #
 
 
-@test("Test that file passes CF convention")
+@test("CF conventions", "Test compliance with the CF metadata conventions.")
 class TestCFConvention(Test):
     def run(self):
         from cfchecker import cfchecks  # pylint: disable=import-outside-toplevel
 
-        standard_names = "CF_STANDARD_NAMES_XML"
-        area_types = "CF_AREA_TYPES_XML"
-        region_names = "CF_REGION_NAMES_XML"
         cf_version = "1.8"
-        env = os.environ
-        if all(key in env for key in (standard_names, area_types, region_names)):
-            inst = cfchecks.CFChecker(
-                silent=True,
-                version=cf_version,
-                cfStandardNamesXML=env[standard_names],
-                cfAreaTypesXML=env[area_types],
-                cfRegionNamesXML=env[region_names],
-            )
-        else:
-            inst = cfchecks.CFChecker(silent=True, version=cf_version)
+        inst = cfchecks.CFChecker(
+            silent=True,
+            version=cf_version,
+            cfStandardNamesXML=CF_STANDARD_NAMES_XML,
+            cfAreaTypesXML=CF_AREA_TYPES_XML,
+            cfRegionNamesXML=CF_REGION_NAMES_XML,
+        )
         result = inst.checker(str(self.filename))
         for key in result["variables"]:
             for level, error_msg in result["variables"][key].items():
